package com.muyu.common.rabbit.producer;

import com.muyu.common.core.domain.Result;
import com.muyu.common.rabbit.constants.RabbitmqConstants;
import lombok.AllArgsConstructor;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.MessageProperties;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;

import java.util.UUID;

/**
 * @ClassName: RabbitMQProducer
 * @Description: rabbitmq生产者
 */
@Component
@AllArgsConstructor
@Log4j2
public class RabbitMQProducerUtil {
    //redis工具类对象

    //rabbit
    private final RabbitTemplate rabbitTemplate;

    /**
     * 简单模型
     *
     * @param param 传递的消息 （如果是对象需要序列化）
     * @return 结果集
     * 一对一消费，只有一个消费者能接收到
     */
    public Result<?> basicSendMessage(String queueName, Object param, String msg) {

        log.info("【简单模型mq】 : method: 【 basicSendMessage 】 - ages: 【 String : {}, Object : {}, String : {} 】 ---> 【 消息发送中。。。 】", RabbitmqConstants.BASIC_QUEUE_NAME, param, msg);
        // 发送简单模型消息
        // 第一个参数： 绑定规则 相当于 队列名称
        // 第二个参数：消息内容
        rabbitTemplate.convertAndSend(queueName, param, message -> {
             message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
             return message;
        } );

        log.info("【简单模型mq】 : method: 【 basicSendMessage 】- queue: 【 {} 】 ---> 【 消息发送成功 】", RabbitmqConstants.BASIC_QUEUE_NAME);

        return Result.success(msg!=null?msg:"消息发送成功");
    }

    /**
     * Work queue 工作模型
     *
     * @param obj 传递的消息 （如果是对象需要序列化）
     * @return 结果集
     * 多个消费者，你一个我一个分配消费消息，有预取机制，默认公平消费，可配置 能者多劳模式（），谁完成的快，谁多做一点
     */
    public Result<?> workSendMessage(String queueName, Object obj, String msg) {

        log.info("【工作模型mq】 : method: 【 workSendMessage 】 - ages: 【 String : {}, Object : {}, String : {} 】 ---> 【 消息发送中。。。 】", queueName, obj, msg);
        // 发送简单模型消息
        // 第一个参数： 绑定规则 相当于 队列名称
        // 第二个参数：消息内容
        rabbitTemplate.convertAndSend(queueName, obj, message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        } );

        log.info("【工作模型mq】 : method: 【 workSendMessage 】- queue: 【 {} 】 ---> 【 消息发送成功 】", queueName);

        return Result.success("消息发送成功");
    }

    /**
     * Publish/Subscribe  发布订阅者模型
     * 多个消费者，多个消费者可以同时接收到消息  有交换机  类型  fanout
     * @param exchange 交换机名称
     * @param obj      发送的消息Object
     * @param msg      响应的内容
     * @return 结果集
     */
    public Result<?> publishSubscribeSendMessage(String exchange, Object obj, String msg) {

        log.info("【订阅模型mq】 : method: 【 workSendMessage 】 - ages: 【 String : {}, Object : {}, String : {} 】 ---> 【 消息发送中。。。 】", exchange, obj, msg);
        // 发送简单模型消息
        // 第一个参数： exchange 交换机的名称
        // 第二个参数： 绑定规则 发布订阅者模型 不写  默认 "" 只要绑定就行 不需要规则
        // 第三个参数：消息内容
        rabbitTemplate.convertAndSend(exchange, "", obj, message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        } );

        log.info("【订阅模型mq】 : method: 【 workSendMessage 】- exchange: 【 {} 】 ---> 【 消息发送成功 】", exchange);

        return Result.success("消息发送成功");
    }

    /**
     * Routing路由模型
     * 使用的是 Direct 类型的交换机，会将接收到的消息根据 规则 路由到指定的Queue(队列)，因此称为路由模式
     *
     * @param exchange 交换机名称
     * @param rule     绑定规则  一个字符串即可
     * @param obj      发送的消息Object
     * @param msg      响应的内容
     * @return 结果集
     */
    public Result<?> routingSendMessage(String exchange, String rule, Object obj, String msg) {

        log.info("【路由模型mq】 : method: 【 workSendMessage 】 - ages: 【 String : {}, Object : {}, String : {} 】 ---> 【 消息发送中。。。 】", exchange, obj, msg);
        // 发送简单模型消息
        // 第一个参数： 绑定规则 相当于 队列名称
        // 第二个参数：消息内容
        rabbitTemplate.convertAndSend(exchange, rule, obj, message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        } );

        log.info("【路由模型mq】 : method: 【 workSendMessage 】- exchange: 【 {} 】 ---> 【 消息发送成功 】", exchange);

        return Result.success("消息发送成功");
    }


    /**
     * Topic主题模型模型
     * 使用的是 topic 类型的交换机
     *
     * @param exchange 交换机名称
     * @param rule     绑定规则  可以绑定多个单词以 . 拼接 也可以使用 #(匹配 零个 一个 或 多个 单词) 或 *(匹配 一个 单词) 通配符（例如：name.msg, *.msg, age.# ）
     * @param obj      发送的消息Object
     * @param msg      响应的内容
     * @return 结果集
     */
    public Result<?> topicSendMessage(String exchange, String rule, Object obj) {

        log.info("【主题模型mq】 : method: 【 workSendMessage 】 - ages: 【 String : {}, Object : {} 】 ---> 【 消息发送中。。。 】", exchange, obj);
        // 发送简单模型消息
        // 第一个参数： 绑定规则 相当于 队列名称
        // 第二个参数：消息内容
        rabbitTemplate.convertAndSend(exchange, rule, obj, message -> {
            message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
            return message;
        } );

        log.info("【主题模型mq】 : method: 【 workSendMessage 】- exchange: 【 {} 】 ---> 【 消息发送成功 】", exchange);

        return Result.success(obj,"消息发送成功");
    }


    /**
     * 延迟队列模型
     * @param param 传输内容
     * @param delayTime 延迟时间
     * @return 结果集
     */
    public Result<?> delayedSendMessage(Long delayTime,  Object param) {
        log.info("【延迟队列模型】 : method: 【 delayedSendMessage 】 消息内容:{}---> 【 消息发送中。。。 】",param);

        rabbitTemplate.convertAndSend(RabbitmqConstants.DELAYED_EXCHANGE_NAME, RabbitmqConstants.DELAYED_ROUTING_KEY,param, message -> {
            MessageProperties messageProperties = message.getMessageProperties();
            messageProperties.setMessageId(UUID.randomUUID().toString());
            messageProperties.setDelayLong(delayTime);
            return message;
        });
        log.info("【延迟队列模型】 : method: 【 delayedSendMessage 】 消息内容:{}---> 【 消息发送成功 】",param);

        return Result.success(param,"消息发送成功");

    }

}
