package com.muyu.template.config;
import cn.hutool.json.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.muyu.cache.MessageTemplateCacheService;
import com.muyu.cache.MessageTemplateTypeCacheService;
import com.muyu.cache.SysCarCacheService;
import com.muyu.cache.TemplateCacheService;
import com.muyu.common.domain.MessageTemplate;
import com.muyu.common.domain.MessageTemplateType;
import com.muyu.common.domain.SysCar;
import com.muyu.common.domain.Template;
import com.muyu.common.domain.resp.SysCarVo;
import com.muyu.common.redis.service.RedisService;

import lombok.extern.log4j.Log4j2;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 *
 * @author liuxinyue
 * @Package：com.muyu.mqtt.configure
 * @Project：cloud-server
 * @name：MqttConfigure
 * @Date：2024/9/28 16:10
 */
@Log4j2
@Component
public class MqttConfigure {

    @Autowired
    private RedisService redisService;

    @Autowired
    private SysCarCacheService service;

    @Autowired
    private MessageTemplateCacheService messageTemplateCacheService;

    @Autowired
    private MessageTemplateTypeCacheService messageTemplateTypeCacheService;

    @Autowired
    private KafkaProducer kafkaProducer;

    @PostConstruct
    public void MQTTMonitoring(){

        String topic        = "car";
        int qos             = 2;
        String broker       = "tcp://47.101.53.251:1883";
        String clientId     = "lxy";
        try {
            MqttClient sampleClient = new MqttClient(broker, clientId);
            MqttConnectOptions connOpts = new MqttConnectOptions();
            //是否清空session
            connOpts.setCleanSession(false);
            log.info("Connecting to broker: " + broker);
            //连接
            sampleClient.connect(connOpts);
            sampleClient.subscribe(topic,0);
            sampleClient.setCallback(new MqttCallback() {
                //连接丢失(报错)
                @Override
                public void connectionLost(Throwable throwable) {
                    log.error("error:"+throwable.getMessage());
                }
                //消息已经接收到
                @Override
                public void messageArrived(String s, MqttMessage mqttMessage) throws Exception {
                    // 将MQTT消息转换为字符串
                    String messageContent = new String(mqttMessage.getPayload());
                    // 解析JSON字符串
                    JSONObject jsonObject = new JSONObject(messageContent);
                    // 从JSON对象中获取"msg"字段的值
                    String msgValue = jsonObject.getStr("msg");
                    log.info("接收到的报文为:"+msgValue);
                    messageParsing(msgValue);
                }
                //交付完成
                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {

                }
            });
        } catch(MqttException me) {
            System.out.println("reason "+me.getReasonCode());
            System.out.println("msg "+me.getMessage());
            System.out.println("loc "+me.getLocalizedMessage());
            System.out.println("cause "+me.getCause());
            System.out.println("excep "+me);
            me.printStackTrace();
        }
    }

    public JSONObject messageParsing(String templateMessage) {
        //给一个JSON对象
        JSONObject jsonObject = new JSONObject();
        //先截取出VIN码 然后根据VIN码查询这个车属于什么类型
        if (templateMessage.length() < 18) {
            throw new RuntimeException("The vehicle message is incorrect");
        }
        //将报文进行切割
        String[] hexArray = templateMessage.split(" ");
        StringBuilder result = new StringBuilder();
        for (String hex : hexArray) {
            int decimal = Integer.parseInt(hex, 16);
            result.append((char) decimal);
        }
        //取出VIN码
        String carVin = result.substring(1, 18 );
        log.info("carVin码为:" + carVin);
        //根据VIN码获取车辆信息
        SysCar carByVin = null;
        List<SysCarVo> carList = service.get("carList");
        if(carList==null){
            throw new RuntimeException("Redis未获取到车辆数据!!!");
        }else{
            // 使用 HashMap 存储车信息，以 VIN 作为键
            Map<String, SysCarVo> carMap = carList.stream()
                    .collect(Collectors.toMap(SysCarVo::getCarVin, Function.identity()));
            carByVin = carMap.get(carVin);
        }

        log.info("车辆信息为:" + carByVin);
        //对应车辆所对应的报文模版
        Integer templateId = carByVin.getTemplateId();
        List<MessageTemplateType> messageTemplateTypes=null;
        String redisKey = "messageTemplateTypeList";
        //key存在
        Boolean b = redisService.hasKey(redisKey);
        if (b) {
             messageTemplateTypes = messageTemplateTypeCacheService.get(redisKey);
        }else{
            throw new RuntimeException("请先将配置存入Redis!!!");
        }
        //将模版里面有的配置进行循环
        for (MessageTemplateType messageTemplateType : messageTemplateTypes) {
            //开始位置
            Integer startIndex = messageTemplateType.getStartIndex() - 1;
            //结束位置
            Integer endIndex = messageTemplateType.getEndIndex();
            //将每个解析后的字段都存入到JSON对象中
            jsonObject.put(messageTemplateType.getMessageField(), result.substring(startIndex, endIndex));
        }
        log.info("解析后的报文是:" + jsonObject);
        sendKafka(jsonObject);
        log.info("发送kafka成功");
        return jsonObject;
    }

    //kafka发送消息
    public void sendKafka(JSONObject jsonObject){
        ProducerRecord<String, String> stringStringProducerRecord = new ProducerRecord<>("four_car", jsonObject.toString());
        kafkaProducer.send(stringStringProducerRecord);
        log.info("kafka发送成功");
    }

}
